﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.


//LoadStrTable(PathAndFile, Separator)	 loads a <separator> delimited table as a 2D text wave.
//Pass an empty String to PathAndFile to bring up the file menu.
//Possible separators: ",", ";", Num2Str(9)=Tab, Num2Str(10)=LineFeed, Num2Str(13)=CarriageReturn


Function LoadStrTable(PathAndFile, Separator, [OutName])			
	String PathAndFile,Separator
	String OutName
	
	Variable RefNum
	Variable Tabs, Lines, len
	String buffer
	
	If(ParamIsDefault(OutName))
		OutName = "T_OutTable"
	Endif

	Open /R RefNum as PathAndFile
	if (refNum == 0)
		return -1						// User canceled
	endif
	
	Tabs = 0
	Lines = 0

	Make/o/n=1/t/FREE w_lines			//text wave containing all the lines


//Read lines and determine Table size	
	Do
		FReadLine refNum, buffer
		len = strlen(buffer)
		if (len == 0)
			break						// End loop when line is empty
		endif
				
		Tabs = Max(ItemsInList(buffer,Separator), Tabs)		//Calculate max. number of columns
		
		w_lines[lines] = buffer
	
		Lines +=1
		redimension/n=(Lines+1) w_lines		//increase table size for next line
	While (1)
	
//	Printf "Loaded Table %s with %g lines and %g columns.\r", PathAndFile, lines, tabs					//Debug only
	
	Make/o/t/FREE/n=(lines, tabs-1) w_out
	

//Assemble table	
	MultiThread w_out[][]=StringFromList(q,w_lines[p],Separator)
	
	Duplicate/o w_out, $OutName
	
	return 0
End


/////////////////////////////////////////////////////////////


Function/S DoOpenMultiFileDialog(fileFilters)
	String fileFilters
	Variable refNum
	String message = "Select one or more files"
	String outputPaths

	
	fileFilters += "All Files:.*;"

	Open /D /R /MULT=1 /F=fileFilters /M=message refNum
	outputPaths = S_fileName
	
	
	return outputPaths		// Will be empty if user canceled
End

//////////////////////////////



Function MultiExperimentDataLoader(objNames,[objTypes])				//Loads named objects from Igor .pxp files into a named data folder
	String objNames 
	Variable objTypes		//LoadFlag: 1=Waves, 2=Numeric Variables, 4=String Variables; May add bitwise

	String allFiles
	String PathAndFile, pathStr, fileStr, saveStr, FileName
	Variable numFilesSelected, ii, pointpos, len
	DFREF SaveDFR=GetDataFolderDFR(), currentDFR
		
	
	allFiles = DoOpenMultiFileDialog("Igor Pro packed experiment:*.pxp;")
	
	If(ParamIsDefault(objTypes))
		objTypes=1
	EndIf
	
	If(strlen(allFiles)==0)
		Print "MultiExperimentDataLoader cancelled"
		return -1
	EndIf
	
	numFilesSelected = ItemsInList(AllFiles, "\r")		//number of files loaded
	
	For(ii=0;ii<numFilesSelected;ii++)
	
		pathandfile = StringFromList(ii, AllFiles, "\r")
		Printf "(%d) Loading data from %s at %s\r", ii, pathandfile, Time()
		
		len=strlen(pathandfile)
		PointPos = strsearch(pathandfile, ":", len, 1)
		PathStr =  pathandfile[0,PointPos-1]
		fileStr = pathandfile[Pointpos+1,len-1]
		
		len=strlen(fileStr)
		PointPos = strsearch(fileStr, ".", len, 1)
		FileName=fileStr[0,PointPos-1]
		
		NewDataFolder/o/s $FileName				//make & set new data folder
		currentDFR=GetDataFolderDFR()
	


		LoadData/Q/O/J=objNames/L=(objTypes) PathAndFile
		SetDataFolder SaveDFR
	EndFor
	
	
End



